@@ -13,6 +13,9 @@
 #include <initializer_list>
 
 #include <leatherman/dynamic_library/dynamic_library.hpp>
+#ifndef _WIN32
+    #include <sys/types.h>
+#endif
 
 namespace leatherman {  namespace ruby {
 
@@ -41,6 +44,11 @@ namespace leatherman {  namespace ruby {
      * See MRI documentation.
      */
     typedef intptr_t SIGNED_VALUE;
+    /**
+     * See MRI documentation.
+     */
+    using LONG_LONG = int64_t;
+    using ULONG_LONG = uint64_t;
     /**
      * See MRI documentation.
      */
@@ -51,6 +59,12 @@ namespace leatherman {  namespace ruby {
      */
     typedef void * rb_encoding_p;
 
+    #ifdef _WIN32
+        typedef int rb_pid_t;
+    #else
+        typedef pid_t rb_pid_t;
+    #endif
+
     /**
      * Macro to cast function pointers to a Ruby method.
      */
@@ -67,6 +81,17 @@ namespace leatherman {  namespace ruby {
         explicit library_not_loaded_exception(std::string const& message);
     };
 
+    /**
+     * Exception thrown when Ruby to C type conversions fail.
+     */
+    struct invalid_conversion : std::runtime_error {
+        /**
+         * Constructs an invalid_conversion exception.
+         * @param message The exception message.
+         */
+        explicit invalid_conversion(std::string const& message);
+    };
+
     /**
      * Contains utility functions and the pointers to the Ruby API.
      */
@@ -98,7 +123,7 @@ namespace leatherman {  namespace ruby {
 
         /**
          * Gets the Ruby API instance.
-         * Throws a runtime_error if the API instance can't be created.
+         * Throws a library_not_loaded_exception if the API instance can't be created.
          * @return Returns the Ruby API instance.
          */
         static api& instance();
@@ -209,11 +234,11 @@ namespace leatherman {  namespace ruby {
         /**
          * See MRI documentation.
          */
-        VALUE (* const rb_num2ulong)(VALUE);
+        ULONG_LONG (* const rb_num2ull)(VALUE);
         /**
          * See MRI documentation.
          */
-        SIGNED_VALUE (* const rb_num2long)(VALUE);
+        LONG_LONG (* const rb_num2ll)(VALUE);
         /**
          * See MRI documentation.
          */
@@ -359,6 +384,11 @@ namespace leatherman {  namespace ruby {
          */
         VALUE (* const rb_require)(char const*);
 
+        /**
+         * Intern MRI method. We're being naughty
+         */
+        VALUE(* const rb_last_status_set)(int, rb_pid_t);
+
         /**
          * See MRI documentation.
          */
@@ -382,15 +412,11 @@ namespace leatherman {  namespace ruby {
         /**
          * See MRI documentation.
          */
-        VALUE* const rb_cFixnum;
+        VALUE* const rb_cInteger;
         /**
          * See MRI documentation.
          */
         VALUE* const rb_cFloat;
-        /**
-         * See MRI documentation.
-         */
-        VALUE* const rb_cBignum;
 
         /**
          * See MRI documentation.
@@ -423,6 +449,14 @@ namespace leatherman {  namespace ruby {
          */
         std::vector<std::string> get_load_path() const;
 
+        /**
+         * Converts a Ruby number into a size_t integer, with checking for overflow.
+         * Throws an invalid_conversion if overflow is detected.
+         * @param v The Ruby value to convert.
+         * @return Returns the Ruby value as a size_t integer.
+         */
+        size_t num2size_t(VALUE v) const;
+
         /**
          * Converts a Ruby value into a C++ string.
          * @param v The Ruby value to convert.
@@ -561,6 +595,13 @@ namespace leatherman {  namespace ruby {
          */
         bool is_fixednum(VALUE value) const;
 
+         /**
+         * Determines if the given value is a big number (Bignum).
+         * @param value The value to check.
+         * @return Returns true if the given value is a fixed number (Fixnum) or false if it is not.
+         */
+        bool is_bignum(VALUE value) const;
+
         /**
          * Determines if the given value is a float.
          * @param value The value to check.
@@ -588,6 +629,7 @@ namespace leatherman {  namespace ruby {
 
         /**
          * Get the length of a ruby array.
+         * Throws an invalid_conversion if teh array length can not be represented by a long.
          * @return Returns the length of the array.
          */
         long array_len(VALUE array) const;
